﻿// Copyright © 2013, 2016, Oracle and/or its affiliates. All rights reserved.
//
// MySQL for Visual Studio is licensed under the terms of the GPLv2
// <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
// MySQL Connectors. There are special exceptions to the terms and
// conditions of the GPLv2 as it is applied to this software, see the
// FLOSS License Exception
// <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA

using System;
using System.Text;
using Xunit;

namespace MySql.Parser.Tests.Other
{
  public class Signal
  {
    [Fact]
    public void Signal_51()
    {
      string result = Utility.ParseSql(
        @"
CREATE PROCEDURE p (pval INT)
BEGIN
  DECLARE specialty CONDITION FOR SQLSTATE '45000';
  IF pval = 0 THEN
    SIGNAL SQLSTATE '01000';
  ELSEIF pval = 1 THEN
    SIGNAL SQLSTATE '45000'
      SET MESSAGE_TEXT = 'An error occurred';
  ELSEIF pval = 2 THEN
    SIGNAL specialty
      SET MESSAGE_TEXT = 'An error occurred';
  ELSE
    SIGNAL SQLSTATE '01000'
      SET MESSAGE_TEXT = 'A warning occurred', MYSQL_ERRNO = 1000;
    SIGNAL SQLSTATE '45000'
      SET MESSAGE_TEXT = 'An error occurred', MYSQL_ERRNO = 1001;
  END IF;
END;", true, new Version(5, 1, 0));
      Assert.True(result.IndexOf("missing 'colon'", StringComparison.OrdinalIgnoreCase) != -1);
    }

    [Fact]
    public void Signal_1_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p (pval INT)
BEGIN
  DECLARE specialty CONDITION FOR SQLSTATE '45000';
  IF pval = 0 THEN
    SIGNAL SQLSTATE '01000';
  ELSEIF pval = 1 THEN
    SIGNAL SQLSTATE '45000'
      SET MESSAGE_TEXT = 'An error occurred';
  ELSEIF pval = 2 THEN
    SIGNAL specialty
      SET MESSAGE_TEXT = 'An error occurred';
  ELSE
    SIGNAL SQLSTATE '01000'
      SET MESSAGE_TEXT = 'A warning occurred', MYSQL_ERRNO = 1000;
    SIGNAL SQLSTATE '45000'
      SET MESSAGE_TEXT = 'An error occurred', MYSQL_ERRNO = 1001;
  END IF;
END;
", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_2_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p (divisor INT)
BEGIN
  IF divisor = 0 THEN
    SIGNAL SQLSTATE '22012';
  END IF;
END;", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_3_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p (divisor INT)
BEGIN
  DECLARE divide_by_zero CONDITION FOR SQLSTATE '22012';
  IF divisor = 0 THEN
    SIGNAL divide_by_zero;
  END IF;
END;", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_4_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p (pval INT)
BEGIN
  DECLARE no_such_table CONDITION FOR 1051;
  SIGNAL no_such_table;
END;
", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_5_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p (divisor INT)
BEGIN
  DECLARE my_error CONDITION FOR SQLSTATE '45000';
  IF divisor = 0 THEN
    BEGIN
      DECLARE my_error CONDITION FOR SQLSTATE '22012';
      SIGNAL my_error;
    END;
  END IF;
  SIGNAL my_error;
END;", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_6_55()
    {
      Utility.ParseSql(
        @"
CREATE PROCEDURE p ()
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION
  BEGIN
    SIGNAL SQLSTATE VALUE '99999'
      SET MESSAGE_TEXT = 'An error occurred';
  END;
  DROP TABLE no_such_table;
END;", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_7_55()
    {
      Utility.ParseSql(
        @"
CREATE FUNCTION f () RETURNS INT
BEGIN
  SIGNAL SQLSTATE '01234';  -- signal a warning
  RETURN 5;
END;", false, new Version(5, 5, 50));
    }

    [Fact]
    public void Signal_8_55()
    {
      // ToDo: MYSQLFORVS-612 - This should be working
      //      Utility.ParseSql(
      //        @"
      //DROP TABLE IF EXISTS xx;
      //CREATE PROCEDURE p ()
      //BEGIN
      //  DECLARE EXIT HANDLER FOR SQLEXCEPTION
      //  BEGIN
      //    SET @error_count = @error_count + 1;
      //    IF @a = 0 THEN RESIGNAL SQLSTATE '45000' SET MYSQL_ERRNO=5; END IF;
      //  END;
      //  DROP TABLE xx;
      //END;
      //SET @error_count = 0;
      //SET @a = 0;
      //SET @@max_error_count = 2;
      //CALL p();
      //SHOW ERRORS;", false, new Version(5, 6, 31));
    }

    [Fact]
    public void Signal_9_55()
    {
      // ToDo: MYSQLFORVS-612 - This should be working
      //      Utility.ParseSql(
      //        @"
      //CREATE FUNCTION f () RETURNS INT
      //BEGIN
      //  RESIGNAL;
      //  RETURN 5;
      //END;
      //CREATE PROCEDURE p ()
      //BEGIN
      //  DECLARE EXIT HANDLER FOR SQLEXCEPTION SET @a=f();
      //  SIGNAL SQLSTATE '55555';
      //END;
      //CALL p();", false, new Version(5, 6, 31));
    }

    [Fact]
    public void Signal_10_55()
    {
      Utility.ParseSql(
        @"CREATE TRIGGER t_bi BEFORE INSERT ON t FOR EACH ROW RESIGNAL;", false, new Version(5, 6, 31));
    }
  }
}
